/*
    SPDX-FileCopyrightText: 2015-2016 Ivan Cukic <ivan.cukic@kde.org>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

#pragma once

#define SQLITE_PLUGIN_CONFIG_KEY "Plugin-org.kde.ActivityManager.Resources.Scoring"
