/*
    SPDX-FileCopyrightText: 2017 Roman Gilg <subdiff@gmail.com>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#pragma once

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(KCM_TOUCHPAD)
